<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 11/09/14
 * Time: 11:29
 */

namespace Folha\Logic\ImportacaoLoteEvento;


use \Folha\Entities\GpeVinculom;
use \Folha\Entities\FpaEventoh;

class ValidadorArquivo
{
    /**
     * ------------------------------------- ATRIBUTO -------------------------------------
     * @var array $infosarquivo
     */
    private $infosarquivo;

    /**
     * ------------------------------------- CONSTRUTOR -------------------------------------
     * @var array $objinfoarquivo
     */
    function __construct($infosarquivo, $nrorg)
    {
        $this->infosarquivo = $infosarquivo;
    }

    /**
     * ------------------------------------- MÉTODOS -------------------------------------
     * VALIDA EVENTO
     * -----------------------------------------------------------------------------------
     * A rotina de importação deve conter uma validação (somente informativa) indicando que o nome do
     * evento na planilha está divergente do nome do evento no sistema.
     * Verificar se o evento existe.
     * -----------------------------------------------------------------------------------
     * @param $nreventom
     * @param $nmeventom
     * @param $nrviculo
     * @param $nmvinculo
     * @param $nrorg
     * @return bool
     * @throws \Exception
     */
    private function validaEvento($nreventom, $nmeventom, $nrviculo, $nmvinculo, $nrorg)
    {
        try {
            /**
             * @var FpaEventoh $obj
             */
            if (!empty($nreventom)) {
                $params = array('P_NREVENTOM' => $nreventom,
                    'P_NRORGPADRAO' => 0,
                    'P_NRORG' => $nrorg);

                $where = "(NRORG = :P_NRORGPADRAO OR NRORG = :P_NRORG) AND NREVENTOH = (SELECT FPA_EVENTOH.NREVENTOH FROM FPA_EVENTOH WHERE NREVENTOM = :P_NREVENTOM AND FPA_EVENTOH.DTMESCOMPETENC = (SELECT MAX(MAX_EVENTOH.DTMESCOMPETENC) FROM FPA_EVENTOH MAX_EVENTOH WHERE (NRORG = :P_NRORGPADRAO OR NRORG = :P_NRORG) AND MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG AND MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM))";
                $obj = Fpaeventoh::getObjectByWhere($where, $params);

            } elseif (!empty($nmeventom)) {
                $params = array('P_NMEVENTOM' => $nmeventom,
                    'P_NRORGPADRAO' => 0,
                    'P_NRORG' => $nrorg);
                $where = '(NRORG = :P_NRORGPADRAO OR NRORG = :P_NRORG) AND NREVENTOH = (SELECT FPA_EVENTOH.NREVENTOH FROM FPA_EVENTOH WHERE UPPER(NMEVENTOH) LIKE UPPER(:P_NMEVENTOM) AND FPA_EVENTOH.DTMESCOMPETENC = (SELECT MAX(MAX_EVENTOH.DTMESCOMPETENC) FROM FPA_EVENTOH MAX_EVENTOH WHERE (NRORG = :P_NRORGPADRAO OR NRORG = :P_NRORG) AND MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG AND MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM))';
                $obj = FpaEventoh::getObjectByWhere($where, $params);
            }

            if ($obj) {
                if(empty($nreventom)){
                    throw new \Exception("O código informado não foi localizado.", 11);
                }elseif (($obj->getNreventom() != $nreventom) || ($obj->getNmeventoh() != $nmeventom)) {
                    throw new \Exception("O código do evento não corresponde ao nome informado para o vínculo $nrviculo - $nmvinculo.", 11);
                }
                return true;

            } else {
                throw new \Exception(" O Evento do vínculo $nrviculo - $nmvinculo não existente.", 10);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * VALIDA VÍNCULO
     * ------------------------------------------------------------------------------------
     * A rotina de importação deve conter uma validação que impeça a conclusão da rotina caso não exista
     * vínculo algum com o respectivo número de matrícula contido na planilha.
     * ------------------------------------------------------------------------------------
     * @param $nrmaticula
     * @param $nmvinculom
     * @param $nrorg
     * @return bool
     * @throws \Exception
     */
    private function validaVinculo($nrvinculom, $nmvinculom, $nrorg)
    {
        $params = array('P_NRORG' => $nrorg,
            'P_NRVINCULOM' => $nrvinculom,
            'P_NMVINCULOM' => $nmvinculom);

        if (!empty($nrvinculom)) {
            $where = 'NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM';
            $objvinculo = GpeVinculom::getObjectByWhere($where, $params);
        }elseif(!empty($nmvinculom)){
            $where = 'NRORG = :P_NRORG AND NMVINCULOM = :P_NMVINCULOM';
            $objvinculo = GpeVinculom::getObjectByWhere($where, $params);
        }

        if(isset($objvinculo) and !empty($objvinculo)){
            return true;
        }else{
            throw new \Exception("Não foi encontrado o vínculo $nrmaticula - $nmvinculom", 10);
        }
    }
    private function validainfo($info)
    {
        try {
            if (!$info) {
                throw new \Exception('Não há eventos no arquivo.', 10);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validar($nrorg)
    {
        try {
            $this->validainfo($this->infosarquivo);
            foreach ($this->infosarquivo as $infoarquivo) {
                try {
                    $evento = $this->validaEvento($infoarquivo['NREVENTOM'], $infoarquivo['NMEVENTOH'], $infoarquivo['NRVINCULOM'], $infoarquivo['NMVINCULOM'], $nrorg);
                    $vinculo = $this->validaVinculo($infoarquivo['NRVINCULOM'], $infoarquivo['NMVINCULOM'], $nrorg);
                } catch (\Exception $e) {
                    if ($e->getCode() == 10) {
                        throw $e;
                    } else {
                        tk_message($e->getMessage());
                    }
                }
            }
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }
} 