<?php
namespace Geral\Util;
/**
 * Created by JetBrains PhpStorm.
 * User: gustavoavila
 * Date: 27/11/12
 * Time: 15:18
 * To change this template use File | Settings | File Templates.
 */
class Util
{

    public static function arrayToEntity(array $fields, $className)
    {
        $entity = new $className();
        foreach ($fields as $property => $value) {
            $property = strtolower($property);
            $set = "set".ucfirst($property);
            if (!property_exists($className, $property))
                throw new \Geral\Exceptions\InvalidEntityFieldException("Field does not exists in entity object.");
            if (substr($property, 0, 2) === 'dt' && $value) {
                $regex = '/\d{2}\/\d{2}\/\d{4} \d{2}:\d{2}:\d{2}/';
                $format = preg_match($regex, $value) ? DateUtil::FORMATO_BRASILEIRO_DATAHORA : DateUtil::FORMATO_BRASILEIRO;
                $entity->$set(DateUtil::getDataDeString($value, $format));
            } else {
                $entity->$set($value);
            }
        }

        return $entity;
    }

    /**
     * Converte uma string na codificacao iso-8859-1 para utf-8
     * @param $str
     * @return string
     */
    public static function convertISOToUTF8($str)
    {
        $encoding = mb_detect_encoding($str, 'utf-8, iso-8859-1, ascii', true);
        if (strcasecmp($encoding, 'UTF-8') !== 0) {
          $str = iconv($encoding, 'utf-8', $str);
        }

        return $str;
    }

    /**
     * Le e retorna string contendo a ultima linha de um arquivo.
     * @param $filePath string Caminho completo para o arquivo.
     * @return string
     */
    public static function readFileLastLine($filePath)
    {
           $fp = @fopen($filePath, "r");
           $pos = -1;
           $line = " ";
           while ($line != "\n") {
                 fseek($fp, $pos, SEEK_END);
                 $line = fgetc($fp);
                 $pos = $pos - 1;
           }
           $line = fgets($fp);
           fclose($fp);
           return $line;
    }

    /**
     * Converte um string com mascara de real currency em Float
     * @param $value
     * @return mixed
     */
    public static function currencyToFloat($string)
    {
        return (float)str_replace(',', '.', str_replace('.', '', $string));
    }

    /**
     * Converte um string decimal em Float
     * @param $value
     * @return mixed
     */
    public static function decimalToFloat($string)
    {
        return (float)str_replace(',', '.', $string);
    }

    /**
     * @param $string string Palavra.
     * @return string
     */
    public static function removeAcentos($string)
    {
        /**
         * A função "strtr" substitui os caracteres acentuados pelos não acentuados.
         */
        $acentos = array(
            'À','Á','Ã','Â', 'à','á','ã','â',
            'Ê', 'É',
            'Í', 'í',
            'Ó','Õ','Ô', 'ó', 'õ', 'ô',
            'Ú','Ü',
            'Ç', 'ç',
            'é','ê',
            'ú','ü',
            'Ñ','ñ'
        );
        $remove_acentos = array(
            'A', 'A', 'A', 'A', 'a', 'a', 'a', 'a',
            'E', 'E',
            'I', 'i',
            'O', 'O','O', 'o', 'o','o',
            'U', 'U',
            'C', 'c',
            'e', 'e',
            'u', 'u',
            'N','n'
        );

        return str_replace($acentos, $remove_acentos, $string);
    }

    /**
     * @param $val  mixed     - Valor a mascarar
     * @param $mask string    - Mascara
     *          # <Caractere a substituir>
     *          qualquer outro caractere diferente de # nas mascara sera tratado como tal.
     * @param $strict boolean - Se parametro for passado, entao tamanho da string devera condizer com o tamanho de # da mascara;
     * @return bool|string
     */
    public static function mask($val, $mask, $strict = null)
    {
        if($strict && strlen($val) !== strlen(preg_replace("/[^\#]/","",$mask)))
            return false;
        $maskared = '';
        $k = 0;
        for($i = 0; $i<=strlen($mask)-1; $i++)
        {
            if($mask[$i] == '#'){
                 if(isset($val[$k]))
                     $maskared .= $val[$k++];
            }else{
                if(isset($mask[$i]))
                    $maskared .= $mask[$i];
            }
        }
        return $maskared;
    }

    /**
     * Remove a mascara de acordo com o @param $regex
     *
     * @param string $masked_value Valor com mascara.
     * @param string $regex        Expressão regular.
     *
     * @return mixed|NULL
     */
    public static function removeMask($masked_value, $regex){
        $mask = $masked_value;
        if (isset($mask) === true) {
            $value = preg_replace($regex, '', $mask);
            return $value;
        } else {
            return null;
        }
    }

    public static function picturePessoa($tipoPessoa)
    {
        switch($tipoPessoa) {
            case "F":
                return '###.###.###-##';
            case "J":
                return '##.###.###/####-##';
            case "O":
                return '#####';
            default:
                return '####################';
        }
    }

    public static function validaPlacaVeiculo($placa)
    {
        return preg_match("/[A-Z]{3}\d{4}/", $placa);
    }

    public static function removeCharsFromString($string)
    {
        return preg_replace('/[^0-9]/', '', $string);
    }

    /**
     * Remove mascara de campos de inscricao juridica e fisica.
     * @param $string String mascara.
     * @return string
     */
    public static function removeMascaraInscricao($string)
    {
        $patterns[] = "/\//";
        $patterns[] = "/\-/";
        $patterns[] = "/\./";
        $patterns[] = "/\(/";
        $patterns[] = "/\)/";
        $patterns[] = "/\,/";

        return preg_replace($patterns,"","/".$string."/");
    }

    /**
     * Compara se todos os caracteres da string sao iguais ao caractere desejado.
     * @param $string String.
     * @param $char   Caractere a comprar.
     * @return boolean
     */
    public static function isAllCharsEqual($string, $char)
    {
        for($i = 0; $i < strlen($string); $i++) {
            $charAt = substr($string, $i, 1);
            if($charAt !== $char)
                return false;
        }
        return true;
    }

    /**
     * Valida se string é igual a todos caracteres de 0..9 de tamanho = $length.
     * @param $string String a comparar.
     * @param $length Tamanho das strings 0..9.
     * @return boolean
     */
    public static function validaInscricaoNumerosIguais($string, $length)
    {
        for($i = 0; $i <= 9; $i++) {
            $fake = str_pad("", $length, $i);
            if($string === $fake)
                return false;
        }

        return true;
    }

    /**
     * Valida CPF
     * @param $cpf String cpf.
     * @return boolean
     */
    public static function validaCPF($cpf){
        $dv = null;
        $cpf = trim(self::removeCharsFromString($cpf));
        if(empty($cpf) || strlen($cpf) != 11) {
            return false;
        }else{
            if(!self::validaInscricaoNumerosIguais($cpf, 11)) {
                return false;
            }else{
                $sub_cpf = substr($cpf, 0, 9);
                for($i = 0; $i <= 9; $i++) {
                    $dv += (substr($sub_cpf,$i,1) * (10-$i));
                }
                if($dv == 0) {
                    return false;
                }
                $dv = 11 - ($dv % 11);
                if($dv > 9){
                    $dv = 0;
                }
//                if($cpf[9] != $dv) {
//                    return false;
//                }
                if(substr($cpf,9,1) != $dv) {
                    return false;
                }
                $dv *= 2;
                for($i = 0; $i <= 9; $i++) {
                    $dv += (substr($sub_cpf,$i,1) * (11-$i));
                }
                $dv = 11 - ($dv % 11);
                if($dv > 9) {
                    $dv = 0;
                }
                if(substr($cpf,10,1) != $dv) {
                    return false;
                }
                return true;
            }
        }
    }
	
    /**
     * Valida PIS
     * @param $pis String pis.
     * @return boolean
     */
    public static function validaPIS($pis){
        $dv = null;
        $pis = str_pad( ereg_replace('[^0-9]', '', $pis), 11, '0', STR_PAD_LEFT);
        if (strlen($pis) != 11 || intval($pis) == 0) {
           return false;
        } else {
        for ($d = 0, $p = 3, $c = 0; $c < 10; $c++) {
           $d += $pis{$c} * $p;
           $p = ($p < 3) ? 9 : --$p;
        }
        $d = ((10 * $d) % 11) % 10;
        return ($pis{$c} == $d) ? true : false;
       }
    }			
		

    /**
     * Valida CGC(CNPJ)
     * @param $cnpj String codigo.
     * @return boolean
     */
    public static function validaCNPJ($cnpj)
    {
        $sum = null;
        $multiplier = null;
        $cnpj = trim(self::removeCharsFromString($cnpj));
        if(empty($cnpj) || strlen($cnpj) != 14) {
            return false;
        }else{
            if(!self::validaInscricaoNumerosIguais($cnpj, 14)){
                return false;
            }else{
                $rev_cnpj = strrev(substr($cnpj, 0, 12));
                for($i = 0; $i <= 11; $i++) {
                    $i == 0 ? $multiplier = 2 : $multiplier;
                    $i == 8 ? $multiplier = 2 : $multiplier;
                    $multiply = ($rev_cnpj[$i] * $multiplier);
                    $sum = $sum + $multiply;
                    $multiplier++;
                }
                $rest = $sum % 11;
                if($rest == 0 || $rest == 1){
                    $dv1 = 0;
                }else{
                    $dv1 = 11 - $rest;
                }

                $sub_cnpj = substr($cnpj, 0, 12);
                $rev_cnpj = strrev($sub_cnpj.$dv1);
                $sum = null;
                for($i = 0; $i <= 12; $i++) {
                    $i == 0 ? $multiplier = 2 : $multiplier;
                    $i == 8 ? $multiplier = 2 : $multiplier;
                    $multiply = ($rev_cnpj[$i] * $multiplier);
                    $sum = $sum + $multiply;
                    $multiplier++;
                }
                $rest = $sum % 11;
                if($rest == 0 || $rest == 1) {
                    $dv2 = 0;
                }else{
                    $dv2 = 11 - $rest;
                }
                if($dv1 == $cnpj[12] && $dv2 == $cnpj[13]){
                    return true;
                }else{
                    return false;
                }
            }
        }
    }
    
    public static function zeroFill($stringToFill, $width, $pad_type = STR_PAD_LEFT) {
        return str_pad((string)$stringToFill, $width, "0", $pad_type);
    }

}
